import { rankType, ResOpen } from '../../shared/protocols/rank/PtlOpen';
import { rankInfo } from '../../shared/protocols/type';
import { player } from '../../shared/protocols/user/type';
import { Queue } from '../../shared/public/queue';
import { RankClslCross } from './rank_clsl';
//import { RankHbzbJfsCross, RankHbzbJfsLocal, RankHbzbZbsCross } from './rank_hbzb_jfs';
import { RankPower } from './rank_power';
import { RankQjzzd } from './rank_qjzzd';
import { RankSlzd1, RankSlzd2, RankSlzd3, RankSlzd4, RankSlzd5, RankSlzd6 } from './rank_slzd';
import { RankTanXian } from './rank_tanxian';
import { RankTujian } from './rank_tujian';
import { RankWzryCross } from './rank_wzry';
import { RankXszm } from './rank_xszm';
import { RankZccg } from './rank_zccg';
import { sortEd } from '../../module/redis'
import * as util from 'util'
import {RankKfjs} from "./rank_kfjs";

import { clusterRunOnce } from '../../clusterUtils';


export abstract class Rank {
    static list: Partial<{
        qjzzd: RankQjzzd;
        zhanli: RankPower;
        tanxian: RankTanXian;
        //hbzbLocal: RankHbzbJfsLocal;
        // hbzbCross: RankHbzbJfsCross;
        // hbzbZbsCross: RankHbzbZbsCross;
        slzd1: RankSlzd1;
        slzd2: RankSlzd2;
        slzd3: RankSlzd3;
        slzd4: RankSlzd4;
        slzd5: RankSlzd5;
        slzd6: RankSlzd6;
        xszm: RankXszm;
        clslCross: RankClslCross;
        zccg: RankZccg;
        tujian: RankTujian;
        wzryCross: RankWzryCross;
        kfjs: RankKfjs
    }> = {};

    // list: rankInfo[];
    queue = new Queue();
    findKey = 'uid';
    countMaxNum = 50;
    utimeTTL = 60;

    abstract getType(): rankType;
    // abstract compare(other: rankInfo, cur: rankInfo): boolean;
    // abstract compareSort(a: rankInfo, b: rankInfo): number;
    // abstract getRankList(uid: string, obj?: {gud?:player, min?: number, max?:number}): Promise<ResOpen['']>;
    // abstract getValArr(info: rankInfo): number|string; // 运算后的积分值，排名依据

    // 获取rank数据, 注：可能重写，具体根据type类型判断
    async getRankList(uid: string, { min, max }): Promise<{ rankList: rankInfo[]; myRank: rankInfo; }> {
        let conn = G?.server?.uid_connections[uid];
        let data = await this.getRankData(uid);
        let rankList = await this.getRankListRange(min, max);
        let rank = await this.getRankSortByOne(uid);
        let score = await this.getRankScore(uid)
        return {
            rankList: rankList,
            myRank: {
                rank,
                player: data?.player || conn?.gud || {},
                valArr: [ score ]
            }
        };
    }

    // 积分, 排名依据, ，注：可能重写，具体根据type类型判断
    getValArr(info: rankInfo):number|string {
        return info?.valArr[0] || 0
    }

    // 注：可能重写，具体根据type类型判断
    compare(other: rankInfo, cur: rankInfo): boolean {
        return cur.valArr[0] > other.valArr[0];
    }
    
    // rankList排序，注：可能重写，具体根据type类型判断
    compareSort(a: rankInfo, b: rankInfo): number {
        return b.valArr[0] - a.valArr[0];
    }


    // 页码转换为min，max
    static pageToMin (page, offset) {
        let res = {
            min: page * offset,
            max: page * offset + offset
        }
        return res
    }

    get db() {
        return G.mongodb.collection('rankList');
    }
    get type() {
        return this.getType();
    }

    constructor() {
        Rank.list[this.getType()] = this;
        this.cotr();
    }

    get getRedisKey() {
        return util.format('rank:%s:data', this.getType())
    }

    get getRedisKeySort() {
        return util.format('rank:%s:sort', this.getType())
    }

    // 初始化
    async cotr() {
        clusterRunOnce(async () => {
            // redis已存在则不初始化
            if(await this.getRankLen() > 0) return 
            this.db.find({ type: this.type }).toArray().then(listArr => {
                // 写入初始化数据
                listArr = listArr || [];
                listArr.forEach(item => {
                    if(item.idKey && item.data) this.setRankData(item.idKey, item.data)
                })
            });
        })
    }

    // 更新数据与排名
    async setRankData(idKey: string, data: rankInfo) {
        let key = this.getRedisKey
        let keySort = this.getRedisKeySort
        data.utime = G.time
        // data
        G.redis.hSet(key, idKey, data)
        // sort
        let valArr = await this.getValArr(data)
        G.redis.zAdd(keySort, <sortEd>{value: idKey, score: valArr})
        return 
    }

    // 获取单个用户的数据
    async getRankData(idKey: string) {
        let key = this.getRedisKey
        let data = await G.redis.hGet(key, idKey)
        if(!data || data.utime < (G.time - this.utimeTTL)) {
            let type = this.getType()
            let res = await this.db.findOne({isKey: idKey, type})
            if(res) {
                data = G.mongodb.conversionIdObj(res)
                this.setRankData(idKey, data.data)
            }
        }
        if(data) return data
        return undefined || {}
    }

    // 获取单个用户的分数
    async getRankScore(idKey:string): Promise<number> {
        let score = await G.redis.zScore(this.getRedisKeySort, idKey)
        return score || 0
    }   

    // 获取单个用户的排名 *降序
    async getRankSortByOne(idKey: string):Promise<number> {
        let rank = await G.redis.zRevRank(this.getRedisKeySort, idKey)
        return rank === 0 || rank > 0 ? rank: -1;
    }

    // 获取单个指定排名的用户数据 *降序
    async getRankSortDataByOne(rank: number): Promise<rankInfo> {
        let rankList:rankInfo[] = await this.getRankListRange(rank, rank + 1)
        let rankInfo = rankList && rankList.length > 0 ? rankList[0] : {} as rankInfo
        return rankInfo
    }
    
    // 获取排名长度
    async getRankLen() {
        return await G.redis.hLen(this.getRedisKey)
    }

    // 获取指定排名范围的数据 *降序
    async getRankListRange(min:number = 0, max:number = 50): Promise<rankInfo[]> {
        let idKeys = await this.getRankListIdKeyRange(min, max)
        if(idKeys && idKeys.length > 0) {
            let res = await G.redis.hmGet(this.getRedisKey, idKeys)
            res = await this.checkData(res)
            return res.sort(this.compareSort)
        } 
        return []
    }
    
    // 获取指定排名范围的idKey *降序
    async getRankListIdKeyRange(min:number = 0, max: number = 50): Promise<string[]> {
        let idKeys = await G.redis.zRevRange(this.getRedisKeySort, min, max - 1)
        return idKeys || []
    }

    // 获取指定类型的全部rank列表,返回为积分排序后的数组
    async getRankListAll(): Promise<rankInfo[]> {
        let res = await G.redis.hGetAll(this.getRedisKey)
        if(res) {
            // 如果是用户数据，则比对utime，更新数据
            // res = await this.checkData(res)
            return Object.values(res).sort(this.compareSort)
        }
        return []
    }

    // 按排名获取全部的idKey
    async getRankListIdKeyAll(): Promise<string[]> {
        let res = this.getRankListIdKeyRange(0, -1)
        return res
    }

    // 验证数据的过期时间，更新数据
    async checkData(rankList: rankInfo[]): Promise<rankInfo[]> {
        let updateUid = []
        rankList.forEach((value,key) => {
            // 仅针对用户表，公会表暂不考虑
            if(rankList[key].player?.uid && (rankList[key].utime || 0) < (G.time - this.utimeTTL)) {
                // 更新数据
                updateUid.push(rankList[key].player.uid)
            }
        })
        if(updateUid.length > 0) {
            let newUserArr = await G.mongodb.collection('user').find({uid:{$in: updateUid}}).toArray()
            // let newUserArr = await G.redis.gets('user', ...updateUid.map(uid => [uid] as [string])) 
            // let newUserArr = await G.mongodb.collection('user').find({uid:{$in: updateUid}}).toArray()
            newUserArr.forEach(item => {
                let index = rankList.findIndex( x => x.player.uid == item.uid);
                rankList[index].player = item;
                this.setRankData(item.uid, rankList[index]);
            })
        }
        return rankList
    }

    async delRankData(idKey:string) {
        G.redis.hDel(this.getRedisKey, idKey)
        G.redis.zRem(this.getRedisKeySort, idKey)
    }

    // 原逻辑前50(countMaxNum)名才更新数据（上榜），多余的数据会删除。 
    async addNew(info: rankInfo) {
        //this.queue.enqueue(async () => {
            await this.setRankData(info.player[this.findKey], info)
            await this.db.updateOne({ type: this.type, idKey: info.player[this.findKey] }, { $set: { data: info } }, { upsert: true });
            // 删除第50名以后的数据，（排名从0开始计算）
            // let idKeys:string[] = await this.getRankListIdKeyRange(50, -1)
            // idKeys.forEach(idKey => {
            //     this.db.deleteOne({ type: this.type, idKey: idKey });
            //     this.delRankData(idKey)
            // })
        //});
    }

    // 清空相关rank数据
    async clear() {
        this.queue.enqueue(async () => {
            await this.db.deleteMany({ type: this.type });
            G.redis.rawDel(this.getRedisKey)
            G.redis.rawDel(this.getRedisKeySort)
        });
    }
}